/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.location;

import java.util.HashMap;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.kemet.enum_.ECadratType;
import org.kemet.enum_.EJustification;
import org.kemet.enum_.EReturnedMdCCode;
import org.kemet.impl.grammar.IGlobals;
import org.kemet.mdc.Cadrat;
import org.kemet.mdc.Line;
import org.kemet.mdc.LocationManager;
import org.kemet.message.Messages;

/**
 * 
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>location manager tests</b>
 * </p>
 * 
 * @since 0.1
 * 
 */
public class TestMdC extends TestCase implements IGlobals {
	static Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>sequence to test.</b>
	 * </p>
	 */
	private String[] input = new String[] { "N1*N2!G1", "N1%N2*G1", "N1N2G1",
			"N1-N2-G1", "N1*-N2-G1", "N1-N2-", "N1*N2*G1", "N1*N2-G1",
			"N1*N2*X1*A15-G1*N3-G1-N2", "N1*N2?G1", "N1*N2:G1" };

	/**
	 * <p>
	 * <b>sequence 2 to test.</b>
	 * </p>
	 */
	private String[] input2 = new String[] { "N1*P1:N2", "N1:P1-G1",
			"N1*P1:N2!N1:P1-G1", "N1*P1:N2!N1:P1-G1!" };

	/**
	 * <p>
	 * <b>sequence 3 to test.</b>
	 * </p>
	 */
	private String[] input3 = new String[] { "G1*(N1:N2):P1", "G1*(N1:N2:P1",
			"G1*)N1:N2:P1", "G1*((N1:N2)):P1" };

	/**
	 * <p>
	 * <b>supported codes of Mdc.</b>
	 * </p>
	 */
	private HashMap values;

	/**
	 * <p>
	 * <b>initialize the parameters before to execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Before
	public void setUp() {
		values = new HashMap();

		// Supported MdC code
		values.put("-", "ruleIndentVerify");
		values.put("*", "ruleStarVerify");
		values.put(":", "ruleDoublePointVerify");
		values.put("!", "ruleExclamationPointVerify");
		values.put("(", "ruleInputBracketVerify");
		values.put(")", "ruleOutputBracketVerify");
	}

	/**
	 * <p>
	 * <b>test null values.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testNullValues() {
		log.info("**** testNullValues ****");

		try {
			new LocationManager(null, Messages.getString("LocationManager.4"),
					values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			new LocationManager(null, null, values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			new LocationManager(input[3], Messages
					.getString("LocationManager.4"), null);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}
	}

	/**
	 * <p>
	 * <b>test special caracter.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testSpecialCaracter() {
		log.info("**** testSpecialCaracter ****");

		// test if the special caracters are correct
		LocationManager ra = null;

		try {
			ra = new LocationManager(input[0], Messages
					.getString("LocationManager.4"), values);
			assertEquals(ra.getCorrect().getValue(), EReturnedMdCCode.Ok
					.getValue());
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			ra = new LocationManager(input[1], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false); 
		} catch (Exception e) {
			assertTrue(true); 
		}

		try {
			ra = new LocationManager(input[2], Messages
					.getString("LocationManager.4"), values);
			assertEquals(ra.getCorrect().getValue(),
					EReturnedMdCCode.NoSpecialCaracters.getValue());
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			ra = new LocationManager(input[9], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false); 
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			ra = new LocationManager(input[10], Messages
					.getString("LocationManager.4"), values);
			assertEquals(ra.getCorrect().getValue(), EReturnedMdCCode.Ok
					.getValue());
		} catch (Exception e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>fonctionnal tests.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testFonctionnal() {
		log.info("**** testFonctionnal ****");

		try {
			new LocationManager(input[3], Messages
					.getString("LocationManager.4"), values);
			assertTrue(true);
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			new LocationManager(input[4], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			new LocationManager(input[5], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}
	}

	/**
	 * <p>
	 * <b>tests for ident and star codes.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testLocationIndentAndStar() {
		log.info("**** testLocationIndentAndStar ****");

		// tests if information about location generated are correct

		// 1. "-"
		LocationManager ra = new LocationManager();
		ra.getInformationLocation(input[3]);

		assertEquals(Line.size(), 1);

		assertEquals(Line.get(0).getCadratList().size(), 3);

		assertEquals(Cadrat.get(0).getObject(0).getJustification(),
				EJustification.CENTER.getValue());

		assertEquals(Cadrat.get(0).getObject(0).getIndex(), 1);

		// 2. "*"
		ra.getInformationLocation(input[6]);

		assertEquals(Line.size(), 1);

		assertEquals(Line.get(0).getCadratList().size(), 1);
		assertEquals(Cadrat.get(0).getObject(0).getJustification(),
				EJustification.LEFT.getValue());

		// 3. "*", "-"
		ra.getInformationLocation(input[7]);

		assertEquals(Line.size(), 1);
		assertEquals(Line.get(0).getCadratList().size(), 2);

		assertEquals(Cadrat.get(0).getObject(0).getJustification(),
				EJustification.LEFT.getValue());

		assertEquals(Cadrat.get(0).getObject(1).getJustification(),
				EJustification.RIGHT.getValue());

		assertEquals(Cadrat.get(1).getObject(0).getJustification(),
				EJustification.CENTER.getValue());

		// 4. tests for justification
		ra.getInformationLocation(input[8]);

		assertEquals(Line.size(), 1);
		assertEquals(Line.get(0).getCadratList().size(), 4);

		assertEquals(Cadrat.get(0).getObject(0).getJustification(),
				EJustification.NoJustification.getValue());
		assertEquals(Cadrat.get(0).getObject(0).getIndex(), 1);

		assertEquals(Cadrat.get(0).getObject(1).getJustification(),
				EJustification.NoJustification.getValue());
		assertEquals(Cadrat.get(0).getObject(1).getIndex(), 2);

		assertEquals(Cadrat.get(1).getObject(0).getJustification(),
				EJustification.LEFT.getValue());

		assertEquals(Cadrat.get(2).getObject(0).getJustification(),
				EJustification.CENTER.getValue());
		assertEquals(Cadrat.get(3).getObject(0).getJustification(),
				EJustification.CENTER.getValue());

		// 5. tests for associated signs
		assertEquals(Cadrat.get(0).getSign(0), "N1");
		assertEquals(Cadrat.get(0).getSign(1), "N2");
		assertEquals(Cadrat.get(1).getSign(0), "G1");
	}

	/**
	 * <p>
	 * <b>tests for double point code.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testLocationDoublePoint() {
		log.info("**** testLocationDoublePoint ****");

		LocationManager ra = new LocationManager();
		ra.getInformationLocation(input2[0]);

		assertEquals(Line.size(), 1);
		assertEquals(Line.get(0).getCadratList().size(), 2);

		assertEquals(Cadrat.get(0).getType(), ECadratType.upHalf.getValue());
		assertEquals(Cadrat.get(1).getType(), ECadratType.downHalf.getValue());

		ra = new LocationManager();
		ra.getInformationLocation(input2[1]);

		assertEquals(Line.size(), 1);
		assertEquals(Line.get(0).getCadratList().size(), 3);

		assertEquals(Cadrat.get(0).getType(), ECadratType.upHalf.getValue());
		assertEquals(Cadrat.get(1).getType(), ECadratType.downHalf.getValue());

		assertEquals(Cadrat.get(2).getType(), ECadratType.full.getValue());
	}

	/**
	 * <p>
	 * <b>tests for end of line.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testLocationEndOfLine() {
		log.info("**** testLocationEndOfLine ****");

		LocationManager ra;

		try {
			ra = new LocationManager(input2[2], Messages
					.getString("LocationManager.4"), values);
			assertTrue(true);
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			ra = new LocationManager(input2[3], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		ra = new LocationManager();
		ra.getInformationLocation(input2[2]);

		assertEquals(Line.size(), 2);

		Cadrat c = (Cadrat) Line.get(0).getCadratList().get(0);
		assertEquals(c.getType(), ECadratType.upHalf.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(1);
		assertEquals(c.getType(), ECadratType.downHalf.getValue());

		c = (Cadrat) Line.get(1).getCadratList().get(2);
		assertEquals(c.getType(), ECadratType.full.getValue());

		c = (Cadrat) Line.get(1).getCadratList().get(0);
		assertEquals(c.getObject(0).getJustification(), EJustification.CENTER
				.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(0);
		assertEquals(c.getObject(0).getJustification(), EJustification.LEFT
				.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(0);
		assertEquals(c.getObject(1).getJustification(), EJustification.RIGHT
				.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(1);
		assertEquals(c.getObject(0).getJustification(), EJustification.CENTER
				.getValue());
	}

	/**
	 * <p>
	 * <b>tests for bracket code.</b>
	 * </p>
	 * 
	 * @since 0.1
	 */
	@Test
	public void testLocationbracket() {
		log.info("**** testLocationbracket ****");

		LocationManager ra;
		try {
			ra = new LocationManager(input3[0], Messages
					.getString("LocationManager.4"), values);
			assertTrue(true);
		} catch (Exception e) {
			assertTrue(false);
		}

		try {
			ra = new LocationManager(input3[1], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			ra = new LocationManager(input3[2], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		try {
			ra = new LocationManager(input3[3], Messages
					.getString("LocationManager.4"), values);
			assertTrue(false);
		} catch (Exception e) {
			assertTrue(true);
		}

		ra = new LocationManager();
		ra.getInformationLocation(input3[0]);

		assertEquals(Line.size(), 1);
		assertEquals(Line.get(0).getCadratList().size(), 5);

		Cadrat c = (Cadrat) Line.get(0).getCadratList().get(0);
		assertEquals(c.getType(), ECadratType.full.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(1);
		assertEquals(c.getType(), ECadratType.rightquarter.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(2);
		assertEquals(c.getType(), ECadratType.upHalf.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(3);
		assertEquals(c.getType(), ECadratType.downHalf.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(4);
		assertEquals(c.getType(), ECadratType.downHalf.getValue());

		// justification tests

		c = (Cadrat) Line.get(0).getCadratList().get(0);
		assertEquals(c.getObject(0).getJustification(), EJustification.CENTER
				.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(1);
		assertEquals(c.size(), 0);

		c = (Cadrat) Line.get(0).getCadratList().get(2);
		assertEquals(c.getObject(0).getJustification(), EJustification.CENTER
				.getValue());

		c = (Cadrat) Line.get(0).getCadratList().get(4);
		assertEquals(c.getObject(0).getJustification(), EJustification.CENTER
				.getValue());
	}
}
